# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-24 11:14
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0018_remove_answer_value_document_data_migration")]

    operations = [migrations.RemoveField(model_name="answer", name="value_document")]
